@@ -121,7 +121,7 @@ class VersionSetGetter
 	void __add_package_to_result(const string& packageName, FSResult* result) const
 	{
 		// we call getSortedPinnedVersions() to place versions of the same package in the preference order
-		for (auto&& pinnedVersion: __cache.getSortedPinnedVersions(__get_package(packageName)))
+		for (auto&& pinnedVersion: __cache.getSortedVersionsWithPriorities(__get_package(packageName)))
 		{
 			result->emplace_back(std::move(pinnedVersion.version));
 		}
@@ -178,7 +178,10 @@ class VersionSet
 	std::map< string, FSResult > __variables;
 
 	VersionSet(const VersionSet& from, FSResult&& versions)
-		: __filtered(true), __versions(std::move(versions)), __variables(from.__variables)
+		: __binary_getter(from.__binary_getter)
+		, __source_getter(from.__source_getter)
+		, __current_getter(from.__current_getter)
+		, __filtered(true), __versions(std::move(versions)), __variables(from.__variables)
 	{}
  public:
 	explicit VersionSet(const VersionSetGetter* binaryGetter, const VersionSetGetter* sourceGetter)
@@ -582,9 +585,9 @@ class ProvidesFS: public RegexMatchBaseFS
 	bool _match(const Cache&, const SPCV& v) const
 	{
 		auto version = static_cast< const BinaryVersion* >(v);
-		for (const string& virtualPackageName: version->provides)
+		for (const auto& elem: version->provides)
 		{
-			if (_matcher.match(virtualPackageName))
+			if (_matcher.match(elem.packageName))
 			{
 				return true;
 			}
@@ -819,6 +822,36 @@ class BinaryToSourceFS: public TransformFS
 	}
 };
 
+class SourceToBinaryFS: public TransformFS
+{
+ public:
+	SourceToBinaryFS(const Arguments& arguments)
+		: TransformFS(false, arguments)
+	{}
+ protected:
+	FSResult _transform(Context& context, const SPCV& version) const
+	{
+		auto sourceVersion = static_cast< const SourceVersion* >(version);
+
+		FSResult result;
+
+		for (const auto& packageName: sourceVersion->binaryPackageNames)
+		{
+			if (auto binaryPackage = context.cache.getBinaryPackage(packageName))
+			{
+				for (auto binaryVersion: *binaryPackage)
+				{
+					if (binaryVersion->sourceVersionString == sourceVersion->versionString)
+					{
+						context.mergeFsResults(&result, { binaryVersion });
+					}
+				}
+			}
+		}
+		return result;
+	}
+};
+
 class PackageIsInstalledFS: public PredicateFS
 {
  protected:
@@ -900,6 +933,7 @@ CommonFS* constructFSByName(const string& functionName, const CommonFS::Argument
 		CONSTRUCT_FS("source-package", RegexMatchFS(BINARY_VERSION_MEMBER(sourcePackageName), arguments))
 		CONSTRUCT_FS("source-version", RegexMatchFS(BINARY_VERSION_MEMBER(sourceVersionString), arguments))
 		CONSTRUCT_FS("essential", BoolMatchFS(BINARY_VERSION_MEMBER(essential), arguments))
+		CONSTRUCT_FS("important", BoolMatchFS(BINARY_VERSION_MEMBER(important), arguments))
 		CONSTRUCT_FS("installed", BoolMatchFS(BINARY_VERSION_MEMBER(isInstalled()), arguments))
 		CONSTRUCT_FS("description", RegexMatchFS(BINARY_VERSION_MEMBER(description), arguments))
 		CONSTRUCT_FS("package:installed", PackageIsInstalledFS(arguments))
@@ -926,8 +960,12 @@ CommonFS* constructFSByName(const string& functionName, const CommonFS::Argument
 
 		CONSTRUCT_FS("build-depends", DependencyFS<SRT::Type>(SRT::BuildDepends, arguments))
 		CONSTRUCT_FS("build-depends-indep", DependencyFS<SRT::Type>(SRT::BuildDependsIndep, arguments))
+		CONSTRUCT_FS("build-depends-arch", DependencyFS<SRT::Type>(SRT::BuildDependsArch, arguments))
 		CONSTRUCT_FS("build-conflicts", DependencyFS<SRT::Type>(SRT::BuildConflicts, arguments))
 		CONSTRUCT_FS("build-conflicts-indep", DependencyFS<SRT::Type>(SRT::BuildConflictsIndep, arguments))
+		CONSTRUCT_FS("build-conflicts-arch", DependencyFS<SRT::Type>(SRT::BuildConflictsArch, arguments))
+
+		CONSTRUCT_FS("source-to-binary", SourceToBinaryFS(arguments))
 	}
 	else
 	{
@@ -935,8 +973,10 @@ CommonFS* constructFSByName(const string& functionName, const CommonFS::Argument
 
 		CONSTRUCT_FS("reverse-build-depends", ReverseBuildDependencyFS(SRT::BuildDepends, arguments))
 		CONSTRUCT_FS("reverse-build-depends-indep", ReverseBuildDependencyFS(SRT::BuildDependsIndep, arguments))
+		CONSTRUCT_FS("reverse-build-depends-arch", ReverseBuildDependencyFS(SRT::BuildDependsArch, arguments))
 		CONSTRUCT_FS("reverse-build-conflicts", ReverseBuildDependencyFS(SRT::BuildConflicts, arguments))
 		CONSTRUCT_FS("reverse-build-conflicts-indep", ReverseBuildDependencyFS(SRT::BuildConflictsIndep, arguments))
+		CONSTRUCT_FS("reverse-build-conflicts-arch", ReverseBuildDependencyFS(SRT::BuildConflictsArch, arguments))
 
 		CONSTRUCT_FS("binary-to-source", BinaryToSourceFS(arguments))
 	}
@@ -1097,13 +1137,17 @@ void processAliases(string* functionNamePtr, vector< string >* argumentsPtr)
 
 			{ "Zbd", "build-depends" },
 			{ "Zbdi", "build-depends-indep" },
+			{ "Zbda", "build-depends-arch" },
 			{ "Zbc", "build-conflicts" },
 			{ "Zbci", "build-conflicts-indep" },
+			{ "Zbca", "build-conflicts-arch" },
 
 			{ "ZRbd", "reverse-build-depends" },
 			{ "ZRbdi", "reverse-build-depends-indep" },
+			{ "ZRbda", "reverse-build-depends-arch" },
 			{ "ZRbc", "reverse-build-conflicts" },
 			{ "ZRbci", "reverse-build-conflicts-indep" },
+			{ "ZRbca", "reverse-build-conflicts-arch" },
 
 			{ "Ra", "release:archive" },
 			{ "Rn", "release:codename" },
