@@ -335,31 +335,6 @@ vector< pair< string, string > > getPathsOfLocalizedDescriptions(
 	return result;
 }
 
-vector< FileDownloadRecord > getDownloadInfoOfLocalizationIndex(const Config& config,
-		const IndexEntry& entry)
-{
-	return getDownloadInfoFromRelease(config, entry, entry.component + "/i18n/Index");
-}
-
-vector< LocalizationDownloadRecord2 > getDownloadInfoOfLocalizedDescriptions2(
-		const Config& config, const IndexEntry& entry)
-{
-	auto chunkArrays = getChunksOfLocalizedDescriptions(config, entry);
-	auto basePath = getPathOfIndexEntry(config, entry);
-
-	vector< LocalizationDownloadRecord2 > result;
-
-	FORIT(chunkArrayIt, chunkArrays)
-	{
-		LocalizationDownloadRecord2 record;
-		record.localPath = basePath + "_" + join("_", *chunkArrayIt);
-		record.filePart = *(chunkArrayIt->rbegin()); // i.e. 'Translation-xyz' part
-		result.push_back(std::move(record));
-	}
-
-	return result;
-}
-
 vector< LocalizationDownloadRecord3 > getDownloadInfoOfLocalizedDescriptions3(
 		const Config& config, const IndexEntry& entry)
 {
@@ -393,57 +368,113 @@ string getPathOfExtendedStates(const Config& config)
 
 namespace {
 
-bool openingForReadingSucceeds(const string& path, const string& fileType, bool debugging)
+bool openingForReadingSucceeds(const string& path)
 {
 	string openError;
 	File file(path, "r", openError);
-	if (!openError.empty())
-	{
-		if (debugging)
-		{
-			debug2("unable to read %s file '%s': %s", fileType, path, openError);
-		}
-		return false;
-	}
+	return openError.empty();
 	return true;
 }
 
-}
-
-bool verifySignature(const Config& config, const string& path, const string& alias)
+string composeGpgvKeyringOptions(const Config& config)
 {
 	auto debugging = config.getBool("debug::gpgv");
-	if (debugging)
+
+	string result;
+	auto considerKeyring = [&](const string& keyring)
 	{
-		debug2("verifying file '%s'", path);
+		if (debugging) debug2("keyring file is '%s'", keyring);
+		result += format2(" --keyring %s", keyring);
+	};
+
+	considerKeyring(config.getPath("dir::etc::trusted"));
+	for (const auto& keyring: config.getConfigurationPartPaths("dir::etc::trustedparts"))
+	{
+		considerKeyring(keyring);
 	}
 
-	auto keyringPath = config.getString("gpgv::trustedkeyring");
-	if (debugging) debug2("keyring file is '%s'", keyringPath);
-	if (!openingForReadingSucceeds(keyringPath, "keyring", debugging)) return false;
+	return result;
+}
+
+string composeGpgvCommand(const Config& config, const string& path)
+{
+	auto debugging = config.getBool("debug::gpgv");
 
 	auto signaturePath = path + ".gpg";
-	if (debugging)
-	{
-		debug2("signature file is '%s'", signaturePath);
-	}
+	if (debugging) debug2("signature file is '%s'", signaturePath);
 
 	if (!fs::fileExists(signaturePath))
 	{
 		if (debugging) debug2("signature file '%s' doesn't exist, omitting it and assuming self-signed file", signaturePath);
 		signaturePath.clear();
 	}
-	else
+	else if (!openingForReadingSucceeds(signaturePath))
 	{
-		if (!openingForReadingSucceeds(signaturePath, "signature", debugging)) return false;
+		if (debugging) debug2("signature file '%s' is not accessible", signaturePath);
+		signaturePath.clear();
 	}
 
+	return format2("gpgv --status-fd 1 %s %s %s 2>/dev/null || true",
+			composeGpgvKeyringOptions(config), signaturePath, path);
+}
 
-	bool verifyResult = false;
+bool isNotGoodSignature(const string& alias, const string& messageType, const string& message)
+{
+	if (messageType == "EXPSIG")
+	{
+		warn2(__("gpg: '%s': expired signature: %s"), alias, message);
+	}
+	else if (messageType == "EXPKEYSIG")
+	{
+		warn2(__("gpg: '%s': expired key: %s"), alias, message);
+	}
+	else if (messageType == "REVKEYSIG")
+	{
+		warn2(__("gpg: '%s': revoked key: %s"), alias, message);
+	}
+	else if (messageType == "BADSIG")
+	{
+		warn2(__("gpg: '%s': bad signature: %s"), alias, message);
+	}
+	else if (messageType == "ERRSIG")
+	{
+		// gpg was not able to verify signature
+		auto parts = split(' ', message); // <keyid> <pkalgo> <hashalgo> <sig_class> <time> <rc>
+		if (parts.size() != 6)
+		{
+			fatal2(__("gpg: '%s': invalid detailed information string '%s'"), alias, message);
+		}
+		const auto& rc = parts.back();
+
+		if (rc == "9")
+		{
+			warn2(__("gpg: '%s': public key '%s' is not found"), alias, parts[0]);
+		}
+		else if (rc == "4")
+		{
+			warn2(__("gpg: '%s': unknown algorithm '%s/%s'"), alias, parts[1], parts[2]);
+		}
+		else
+		{
+			warn2(__("gpg: '%s': could not verify a signature: %s"), alias, message);
+		}
+	}
+	else if (messageType == "NODATA")
+	{
+		warn2(__("gpg: '%s': empty signature"), alias);
+	}
+	else
+	{
+		return false;
+	}
+	return true;
+}
+
+bool runGpgCommand(const string& gpgCommand, const string& alias, bool debugging)
+{
+	if (debugging) debug2("gpgv command is '%s'", gpgCommand);
 	try
 	{
-		string gpgCommand = string("gpgv --status-fd 1 --keyring ") + keyringPath +
-				' ' + signaturePath + ' ' + path + " 2>/dev/null || true";
 		string openError;
 		File gpgPipe(gpgCommand, "pr", openError);
 		if (!openError.empty())
@@ -452,26 +483,23 @@ bool verifySignature(const Config& config, const string& path, const string& ali
 		}
 
 		smatch m;
-
 		static const sregex gnupgPrefixRegex = sregex::compile("\\[GNUPG:\\] ");
-		string result;
 
-		while (!gpgPipe.getLine(result).eof())
+		string status;
+		while (!gpgPipe.getLine(status).eof())
 		{
-			if (debugging) debug2("fetched '%s' from gpg pipe", result);
-			result = regex_replace(result, gnupgPrefixRegex, "");
+			if (debugging) debug2("fetched '%s' from gpg pipe", status);
+			status = regex_replace(status, gnupgPrefixRegex, "");
 
-			const auto& status = result;
 			if (status.empty())
 			{
-				// no info from gpg at all
 				fatal2(__("gpg: '%s': no information"), alias);
 			}
 
 			static const sregex messageRegex = sregex::compile("(\\w+) (.*)");
 			if (!regex_match(status, m, messageRegex))
 			{
-				continue;
+				continue; // not a status string / message
 			}
 
 			string messageType = m[1];
@@ -479,50 +507,11 @@ bool verifySignature(const Config& config, const string& path, const string& ali
 
 			if (messageType == "GOODSIG")
 			{
-				verifyResult = 1;
+				return true;
 			}
-			else if (messageType == "EXPSIG")
+			else if (isNotGoodSignature(alias, messageType, message))
 			{
-				warn2(__("gpg: '%s': expired signature: %s"), alias, message);
-			}
-			else if (messageType == "EXPKEYSIG")
-			{
-				warn2(__("gpg: '%s': expired key: %s"), alias, message);
-			}
-			else if (messageType == "REVKEYSIG")
-			{
-				warn2(__("gpg: '%s': revoked key: %s"), alias, message);
-			}
-			else if (messageType == "BADSIG")
-			{
-				warn2(__("gpg: '%s': bad signature: %s"), alias, message);
-			}
-			else if (messageType == "ERRSIG")
-			{
-				// gpg was not able to verify signature
-				auto parts = split(' ', message); // <keyid> <pkalgo> <hashalgo> <sig_class> <time> <rc>
-				if (parts.size() != 6)
-				{
-					fatal2(__("gpg: '%s': invalid detailed information string '%s'"), alias, message);
-				}
-				const auto& rc = parts.back();
-
-				if (rc == "9")
-				{
-					warn2(__("gpg: '%s': public key '%s' is not found"), alias, parts[0]);
-				}
-				else if (rc == "4")
-				{
-					warn2(__("gpg: '%s': unknown algorithm '%s/%s'"), alias, parts[1], parts[2]);
-				}
-				else
-				{
-					warn2(__("gpg: '%s': could not verify a signature: %s"), alias, message);
-				}
-			}
-			else if (messageType == "NODATA")
-			{
-				warn2(__("gpg: '%s': empty signature"), alias);
+				return false;
 			}
 		}
 	}
@@ -530,14 +519,55 @@ bool verifySignature(const Config& config, const string& path, const string& ali
 	{
 		warn2(__("unable to verify a signature for the '%s'"), alias);
 	}
+	return false;
+}
 
-	if (debugging)
-	{
-		debug2("the verify result is %u", (unsigned int)verifyResult);
-	}
+}
+
+bool verifySignature(const Config& config, const string& path, const string& alias)
+{
+	const auto debugging = config.getBool("debug::gpgv");
+	if (debugging) debug2("verifying file '%s'", path);
+
+	const auto gpgCommand = composeGpgvCommand(config, path);
+	const bool verifyResult = runGpgCommand(gpgCommand, alias, debugging);
+
+	if (debugging) debug2("the verify result is %u", (unsigned int)verifyResult);
 	return verifyResult;
 }
 
+void verifyReleaseValidityDate(const string& date, const Config& config, const string& releaseAlias)
+{
+	if (date.empty()) {
+		return;
+	}
+
+	struct tm validUntilTm;
+	memset(&validUntilTm, 0, sizeof(validUntilTm));
+	struct tm currentTm;
+
+	auto oldTimeSpec = setlocale(LC_TIME, "C");
+	auto parseResult = strptime(date.c_str(), "%a, %d %b %Y %T UTC", &validUntilTm);
+	setlocale(LC_TIME, oldTimeSpec);
+	if (parseResult) // success
+	{
+		time_t localTime = time(NULL);
+		gmtime_r(&localTime, &currentTm);
+		// sanely, we should use timegm() here, but it's not portable,
+		// so we use mktime() which is enough for comparing two UTC tm's
+		if (mktime(&currentTm) > mktime(&validUntilTm))
+		{
+			bool warnOnly = config.getBool("cupt::cache::release-file-expiration::ignore");
+			(warnOnly ? warn2< string, string > : fatal2< string, string >)
+					(__("the release '%s' has expired (expiry time '%s')"), releaseAlias, date);
+		}
+	}
+	else
+	{
+		warn2(__("unable to parse the expiry time '%s' in the release '%s'"), date, releaseAlias);
+	}
+}
+
 shared_ptr< cache::ReleaseInfo > getReleaseInfo(const Config& config,
 		const string& path, const string& alias)
 {
@@ -620,37 +650,6 @@ shared_ptr< cache::ReleaseInfo > getReleaseInfo(const Config& config,
 		fatal2(__("unable to parse the release '%s'"), alias);
 	}
 
-	{ // checking Valid-Until
-		if (!result->validUntilDate.empty())
-		{
-			struct tm validUntilTm;
-			memset(&validUntilTm, 0, sizeof(validUntilTm));
-			struct tm currentTm;
-
-			auto oldTimeSpec = setlocale(LC_TIME, "C");
-			auto parseResult = strptime(result->validUntilDate.c_str(), "%a, %d %b %Y %T UTC", &validUntilTm);
-			setlocale(LC_TIME, oldTimeSpec);
-			if (parseResult) // success
-			{
-				time_t localTime = time(NULL);
-				gmtime_r(&localTime, &currentTm);
-				// sanely, we should use timegm() here, but it's not portable,
-				// so we use mktime() which is enough for comparing two UTC tm's
-				if (mktime(&currentTm) > mktime(&validUntilTm))
-				{
-					bool warnOnly = config.getBool("cupt::cache::release-file-expiration::ignore");
-					(warnOnly ? warn2< string, string > : fatal2< string, string >)
-							(__("the release '%s' has expired (expiry time '%s')"), alias, result->validUntilDate);
-				}
-			}
-			else
-			{
-				warn2(__("unable to parse the expiry time '%s' in the release '%s'"),
-						result->validUntilDate, alias);
-			}
-		}
-	}
-
 	return result;
 }
 
