@@ -116,28 +116,47 @@ Package* CacheImpl::preparePackage(unordered_map< string, vector< PrePackageReco
 	}
 }
 
-template < bool multiarchAllowedRequired >
-static inline void addSatisfyingPackageVersions(
-		vector<const BinaryVersion*>* result, const Relation& relation,
-		const BinaryPackage* package, const system::State& systemState)
+static bool versionSatisfiesRelation(const BinaryVersion* version, const Relation& relation)
 {
-	if (package)
+	if (relation.isSatisfiedBy(version->versionString))
+	{
+		if (relation.architecture.empty())
+		{
+			return true;
+		}
+		else if (relation.architecture.compare(0, string::npos, "any", 3) == 0)
+		{
+			return version->multiarch.compare(0, string::npos, "allowed", 7) == 0;
+		}
+		else if (relation.architecture.compare(0, string::npos, "native", 6) == 0)
+		{
+			return version->multiarch.compare(0, string::npos, "foreign", 7) == 0;
+		}
+		else
+		{
+			return false; // unsupported / invalid
+		}
+	}
+	else
+	{
+		return false;
+	}
+}
+
+void CacheImpl::addRealPackageSatisfyingVersions(vector<const BinaryVersion*>* result, const Relation& relation) const
+{
+	if (auto package = getBinaryPackage(relation.packageName))
 	{
-		// if such binary package exists
 		for (auto version: *package)
 		{
-			if (relation.isSatisfiedBy(version->versionString))
+			if (versionSatisfiesRelation(version, relation))
 			{
 				if (version->isInstalled() &&
-						systemState.getInstalledInfo(version->packageName)->isBroken() &&
+						systemState->getInstalledInfo(version->packageName)->isBroken() &&
 						relation.relationType != Relation::Types::LiteralyEqual)
 				{
 					continue;
 				}
-				if (multiarchAllowedRequired && version->multiarch != "allowed")
-				{
-					continue;
-				}
 
 				result->push_back(version);
 			}
@@ -145,21 +164,6 @@ static inline void addSatisfyingPackageVersions(
 	}
 }
 
-void CacheImpl::addRealPackageSatisfyingVersions(vector<const BinaryVersion*>* result, const Relation& relation) const
-{
-	auto package = getBinaryPackage(relation.packageName);
-
-	if (relation.architecture.empty())
-	{
-		addSatisfyingPackageVersions<false>(result, relation, package, *systemState);
-	}
-	else if (relation.architecture.compare(0, string::npos, "any", 3) == 0)
-	{
-		addSatisfyingPackageVersions<true>(result, relation, package, *systemState);
-	}
-	// otherwise unsupported
-}
-
 namespace {
 
 bool providesChunkMatchesRelation(const Relation& providesChunk, const Relation& relation)
@@ -295,12 +299,14 @@ static void parseOutKeyValueOptions(vector< string >& tokens, Cache::IndexEntry*
 	for (auto it = openingBracketTokenIt+1; it != closingBracketTokenIt; ++it)
 	{
 		const string& token = *it;
-		auto keyValueDelimiterPosition = token.find('=');
-		if (keyValueDelimiterPosition == string::npos)
-		{
-			fatal2(__("no key-value separator ('=') in the option token '%s'"), token);
-		}
-		entry->options[token.substr(0, keyValueDelimiterPosition)] = token.substr(keyValueDelimiterPosition+1);
+		parse::processSpaceCharSpaceDelimitedStrings(token.begin(), token.end(), ',', [&entry](auto from, auto to) {
+			auto keyValueDelimiterPosition = std::find(from, to, '=');
+			if (keyValueDelimiterPosition == to)
+			{
+				fatal2(__("no key-value separator ('=') in the option token '%s'"), std::string(from, to));
+			}
+			entry->options[string(from, keyValueDelimiterPosition)] = string(keyValueDelimiterPosition+1, to);
+		});
 	}
 
 	tokens.erase(openingBracketTokenIt, closingBracketTokenIt+1);
@@ -507,7 +513,10 @@ void CacheImpl::processIndexEntries(bool useBinary, bool useSource)
 	}
 }
 
-static string getIndexEntryOptionValue(const Cache::IndexEntry& entry, const string& key)
+namespace
+{
+
+string getIndexEntryOptionValue(const Cache::IndexEntry& entry, const string& key)
 {
 	auto it = entry.options.find(key);
 	if (it == entry.options.end())
@@ -520,7 +529,7 @@ static string getIndexEntryOptionValue(const Cache::IndexEntry& entry, const str
 	}
 }
 
-static bool getVerifiedBitForIndexEntry(const Cache::IndexEntry& entry,
+bool getVerifiedBitForIndexEntry(const Cache::IndexEntry& entry,
 		const Config& config, const string& path, const string& alias)
 {
 	auto trustedOptionValue = getIndexEntryOptionValue(entry, "trusted");
@@ -538,23 +547,45 @@ static bool getVerifiedBitForIndexEntry(const Cache::IndexEntry& entry,
 	}
 }
 
-shared_ptr< ReleaseInfo > CacheImpl::getReleaseInfo(const Config& config, const IndexEntry& indexEntry)
+void validateValidUntil(const Config& config, const Cache::IndexEntry& entry,
+		const ReleaseInfo& releaseInfo, const string& alias)
 {
-	auto path = cachefiles::getPathOfMasterReleaseLikeList(config, indexEntry);
-	auto insertResult = releaseInfoCache.insert({ path, {} });
+	if (getIndexEntryOptionValue(entry, "check-valid-until") == "no")
+	{
+		return;
+	}
+
+	cachefiles::verifyReleaseValidityDate(releaseInfo.validUntilDate, config, alias);
+}
+
+shared_ptr<ReleaseInfo> getReleaseInfoNotCached(const Config& config, const Cache::IndexEntry& indexEntry,
+												const string& alias)
+{
+	const auto path = cachefiles::getPathOfMasterReleaseLikeList(config, indexEntry);
+	if (path.empty())
+	{
+		warn2(__("no release file present for '%s'"), alias);
+		return nullptr;
+	}
+	else
+	{
+		auto result = cachefiles::getReleaseInfo(config, path, alias);
+		validateValidUntil(config, indexEntry, *result, alias);
+		result->verified = getVerifiedBitForIndexEntry(indexEntry, config, path, alias);
+		return result;
+	}
+}
+
+}
+
+shared_ptr<ReleaseInfo> CacheImpl::getReleaseInfo(const Config& config, const IndexEntry& indexEntry)
+{
+	const auto alias = indexEntry.uri + ' ' + indexEntry.distribution;
+	auto insertResult = releaseInfoCache.insert({ alias, {} });
 	auto& cachedValue = insertResult.first->second;
 	if (insertResult.second)
 	{
-		auto alias = indexEntry.uri + ' ' + indexEntry.distribution;
-		if (path.empty())
-		{
-			warn2(__("no release file present for '%s'"), alias);
-		}
-		else
-		{
-			cachedValue = cachefiles::getReleaseInfo(config, path, alias);
-			cachedValue->verified = getVerifiedBitForIndexEntry(indexEntry, config, path, alias);
-		}
+		cachedValue = getReleaseInfoNotCached(config, indexEntry, alias);
 	}
 	if (!cachedValue)
 	{
@@ -563,8 +594,7 @@ shared_ptr< ReleaseInfo > CacheImpl::getReleaseInfo(const Config& config, const
 	return shared_ptr< ReleaseInfo > (new ReleaseInfo(*cachedValue));
 }
 
-void CacheImpl::processIndexEntry(const IndexEntry& indexEntry,
-		const ReleaseLimits& releaseLimits)
+void CacheImpl::processIndexEntry(const IndexEntry& indexEntry, const ReleaseLimits& releaseLimits)
 {
 	string indexFileToParse = cachefiles::getPathOfIndexList(*config, indexEntry);
 
@@ -778,16 +808,16 @@ ssize_t CacheImpl::getPin(const Version* version,
 
 string CacheImpl::getLocalizedDescription(const BinaryVersion* version) const
 {
-	const string& sourceHash = !version->descriptionHash.empty() ?
-			version->descriptionHash :
-			HashSums::getHashOfString(HashSums::MD5, version->description);
-
-	auto it = translations.find(sourceHash);
-	if (it != translations.end())
+	const auto& hash = version->descriptionHash;
+	if (!hash.empty())
 	{
-		const TranslationPosition& position = it->second;
-		position.file->seek(position.offset);
-		return position.file->getRecord().chompAsRecord();
+		auto it = translations.find(hash);
+		if (it != translations.end())
+		{
+			const TranslationPosition& position = it->second;
+			position.file->seek(position.offset);
+			return position.file->getRecord().chompAsRecord();
+		}
 	}
 	return version->description;
 }
