@@ -18,15 +18,12 @@
 #ifndef CUPT_INTERNAL_SOLUTION_SEEN
 #define CUPT_INTERNAL_SOLUTION_SEEN
 
-#include <bitset>
 #include <map>
 #include <forward_list>
-#include <cstring>
 
 #include <cupt/cache/binaryversion.hpp>
 #include <cupt/system/resolver.hpp>
 
-#include <internal/nativeresolver/score.hpp>
 #include <internal/nativeresolver/dependencygraph.hpp>
 #include <internal/nativeresolver/cowmap.hpp>
 
@@ -173,7 +170,7 @@ class SolutionStorage
 	bool verifyElement(const PreparedSolution&, dg::Element) const;
 
 	// may include parameter itself
-	static const forward_list<dg::Element>& getConflictingElements(dg::Element);
+	static const vector<dg::Element>& getConflictingElements(dg::Element);
 	bool simulateSetPackageEntry(const PreparedSolution&, dg::Element, dg::Element*) const;
 	void setRejection(PreparedSolution&, dg::Element);
 	void setEmpty(PreparedSolution&, dg::Element);
