@@ -33,15 +33,16 @@
 #include <internal/exceptionlessfuture.hpp>
 
 #include <internal/worker/metadata.hpp>
+#include <internal/worker/temppath.hpp>
 
 namespace cupt {
 namespace internal {
 
-enum class MetadataWorker::IndexType { Packages, PackagesDiff, Localization, LocalizationFile, LocalizationFileDiff };
+enum class MetadataWorker::IndexType { Packages, PackagesDiff, Translation, TranslationDiff };
 
 bool MetadataWorker::__is_diff_type(const IndexType& indexType)
 {
-	return indexType == IndexType::PackagesDiff || indexType == IndexType::LocalizationFileDiff;
+	return indexType == IndexType::PackagesDiff || indexType == IndexType::TranslationDiff;
 }
 
 string MetadataWorker::__get_indexes_directory() const
@@ -89,23 +90,24 @@ string getFilenameExtension(const string& source)
 	}
 };
 
-std::function< string () > generateMovingSub(const string& downloadPath, const string& targetPath)
+std::function< string () > generateMovingSub(const SharedTempPath& downloadPath, const string& targetPath)
 {
-	return [downloadPath, targetPath]() -> string
+	return [downloadPath=downloadPath, targetPath]() mutable -> string
 	{
 		ioi::removeIndexOfIndex(targetPath);
 		if (fs::move(downloadPath, targetPath))
 		{
+			downloadPath.abandon();
 			return "";
 		}
 		else
 		{
-			return format2e(__("unable to rename '%s' to '%s'"), downloadPath, targetPath);
+			return format2e(__("unable to rename '%s' to '%s'"), (string)downloadPath, targetPath);
 		}
 	};
 };
 
-bool generateUncompressingSub(const download::Uri& uri, const string& downloadPath,
+bool generateUncompressingSub(const download::Uri& uri, const SharedTempPath& downloadPath,
 		const string& targetPath, std::function< string () >& sub)
 {
 	auto filenameExtension = getFilenameExtension(uri);
@@ -144,15 +146,13 @@ bool generateUncompressingSub(const download::Uri& uri, const string& downloadPa
 
 		sub = [uncompressorName, downloadPath, targetPath]() -> string
 		{
-			auto uncompressedPath = downloadPath + ".uncompressed";
+			SharedTempPath uncompressedPath { (string)downloadPath + ".uncompressed" };
 			auto uncompressingResult = ::system(format2("%s %s -c > %s",
-					uncompressorName, downloadPath, uncompressedPath).c_str());
-			// anyway, remove the compressed file, ignoring errors if any
-			unlink(downloadPath.c_str());
+					uncompressorName, (string)downloadPath, string(uncompressedPath)).c_str());
 			if (uncompressingResult)
 			{
 				return format2(__("failed to uncompress '%s', '%s' returned the error %d"),
-						downloadPath, uncompressorName, uncompressingResult);
+						(string)downloadPath, uncompressorName, uncompressingResult);
 			}
 			return generateMovingSub(uncompressedPath, targetPath)();
 		};
@@ -255,7 +255,7 @@ bool MetadataWorker::__downloadReleaseLikeFile(download::Manager& downloadManage
 
 	auto alias = indexEntry.distribution + ' ' + name;
 	auto longAlias = indexEntry.uri + ' ' + alias;
-	auto downloadPath = getDownloadPath(targetPath);
+	SharedTempPath downloadPath { getDownloadPath(targetPath) };
 
 	_logger->log(Logger::Subsystem::Metadata, 3, __get_download_log_message(longAlias));
 	{
@@ -373,7 +373,7 @@ static const sregex checksumsLineRegex = sregex::compile(
 bool __download_and_apply_patches(download::Manager& downloadManager,
 		const cachefiles::FileDownloadRecord& downloadRecord,
 		const cachefiles::IndexEntry& indexEntry, const string& baseDownloadPath,
-		const string& diffIndexPath, const string& targetPath,
+		const string& diffIndexPath_, const string& targetPath,
 		Logger* logger)
 {
 	// total hash -> { name of the patch-to-apply, total size }
@@ -388,28 +388,17 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 			' ' + getUriBasename(baseUri);
 	auto baseLongAlias = indexEntry.uri + ' ' + baseAlias;
 
-	auto patchedPath = baseDownloadPath + ".patched";
-
-	auto cleanUp = [patchedPath, diffIndexPath]()
-	{
-		unlink(patchedPath.c_str());
-		unlink(diffIndexPath.c_str());
-	};
-	auto fail = [baseLongAlias, &cleanUp]()
-	{
-		cleanUp();
-		warn2(__("%s: failed to proceed"), baseLongAlias);
-	};
-
 	try
 	{
 		{ // parsing diff index
+			SharedTempPath diffIndexPath(diffIndexPath_);
+
 			string openError;
 			File diffIndexFile(diffIndexPath, "r", openError);
 			if (!openError.empty())
 			{
 				logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
-						piddedFormat2e, "unable to open the file '%s': %s", diffIndexPath, openError);
+						piddedFormat2e, "unable to open the file '%s': %s", string(diffIndexPath), openError);
 			}
 
 			TagParser diffIndexParser(&diffIndexFile);
@@ -418,8 +407,8 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 			while (diffIndexParser.parseNextLine(fieldName, fieldValue))
 			{
 				bool isHistory = fieldName.equal(BUFFER_AND_SIZE("SHA1-History"));
-				bool isPatchInfo = fieldName.equal(BUFFER_AND_SIZE("SHA1-Patches"));
-				if (isHistory || isPatchInfo)
+				bool isDownloadInfo = fieldName.equal(BUFFER_AND_SIZE("SHA1-Download"));
+				if (isHistory || isDownloadInfo)
 				{
 					string block;
 					diffIndexParser.parseAdditionalLines(block);
@@ -461,10 +450,6 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 						piddedFormat2, "failed to find the target hash sum");
 			}
 		}
-		if (unlink(diffIndexPath.c_str()) == -1)
-		{
-			warn2(__("unable to remove the file '%s'"), diffIndexPath);
-		}
 
 		HashSums subTargetHashSums;
 		subTargetHashSums.fill(targetPath);
@@ -472,10 +457,11 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 
 		const string initialSha1Sum = currentSha1Sum;
 
-		if (::system(format2("cp %s %s", targetPath, patchedPath).c_str()))
+		SharedTempPath patchedPath { baseDownloadPath + ".patched" };
+		if (::system(format2("cp %s %s", targetPath, (string)patchedPath).c_str()))
 		{
 			logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
-					piddedFormat2, "unable to copy '%s' to '%s'", targetPath, patchedPath);
+					piddedFormat2, "unable to copy '%s' to '%s'", targetPath, (string)patchedPath);
 		}
 
 		while (currentSha1Sum != wantedHashSum)
@@ -487,7 +473,6 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 				{
 					logger->log(Logger::Subsystem::Metadata, 3, __get_pidded_string(
 							"no matching index patches found, presumably the local index is too old"));
-					cleanUp();
 					return false; // local index is too old
 				}
 				else
@@ -498,20 +483,21 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 			}
 
 			const string& patchName = historyIt->second.first;
-			auto patchIt = patches.find(patchName);
+			auto patchFileName = patchName + ".gz";
+			auto patchIt = patches.find(patchFileName);
 			if (patchIt == patches.end())
 			{
 				logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
 						piddedFormat2, "unable to find a patch entry for the patch '%s'", patchName);
 			}
 
-			string patchSuffix = "/" + patchName + ".gz";
+			string patchSuffix = "/" + patchFileName;
 			auto alias = baseAlias + patchSuffix;
 			auto longAlias = baseLongAlias + patchSuffix;
 			logger->log(Logger::Subsystem::Metadata, 3, __get_download_log_message(longAlias));
 
-			auto unpackedPath = baseDownloadPath + '.' + patchName;
-			auto downloadPath = unpackedPath + ".gz";
+			SharedTempPath downloadPath { baseDownloadPath + '.' + patchFileName };
+			SharedTempPath unpackedPath { baseDownloadPath + '.' + patchName };
 
 			download::Manager::DownloadEntity downloadEntity;
 
@@ -520,7 +506,7 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 					alias, longAlias);
 			downloadEntity.extendedUris.push_back(std::move(extendedUri));
 			downloadEntity.targetPath = downloadPath;
-			downloadEntity.size = (size_t)-1;
+			downloadEntity.size = patchIt->second.second;
 
 			HashSums patchHashSums;
 			patchHashSums[HashSums::SHA1] = patchIt->second.first;
@@ -528,56 +514,50 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 			std::function< string () > uncompressingSub;
 			generateUncompressingSub(patchUri, downloadPath, unpackedPath, uncompressingSub);
 
-			downloadEntity.postAction = [&patchHashSums, &subTargetHashSums,
+			downloadEntity.postAction = [&patchHashSums, &downloadPath,
 					&uncompressingSub, &patchedPath, &unpackedPath]() -> string
 			{
+				if (!patchHashSums.verify(downloadPath))
+				{
+					return __("hash sums mismatch");
+				}
+
 				auto partialDirectory = fs::dirname(patchedPath);
 				auto patchedPathBasename = fs::filename(patchedPath);
 
 				string result = uncompressingSub();
 				if (!result.empty())
 				{
-					return result; // unpackedPath is not yet created
+					return result;
 				}
 
-				if (!patchHashSums.verify(unpackedPath))
-				{
-					result = __("hash sums mismatch");
-					goto out;
-				}
 				if (::system(format2("(cat %s && echo w) | (cd %s && red -s - %s >/dev/null)",
-							unpackedPath, partialDirectory, patchedPathBasename).c_str()))
+							(string)unpackedPath, partialDirectory, patchedPathBasename).c_str()))
 				{
-					result = __("applying ed script failed");
-					goto out;
+					return __("applying ed script failed");
 				}
-				subTargetHashSums.fill(patchedPath);
-			 out:
-				if (unlink(unpackedPath.c_str()) == -1)
-				{
-					warn2e(__("unable to remove the file '%s'"), unpackedPath);
-				}
-				return result;
+				return {};
 			};
 			auto downloadError = downloadManager.download(
 					vector< download::Manager::DownloadEntity >{ downloadEntity });
 			if (!downloadError.empty())
 			{
-				fail();
-				return false;
+				throw std::runtime_error(""); // error message is reported already by download manager
 			}
+
+			subTargetHashSums.fill(patchedPath);
 		}
 
 		if (!fs::move(patchedPath, targetPath))
 		{
 			logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
-					piddedFormat2e, "unable to rename '%s' to '%s'", patchedPath, targetPath);
+					piddedFormat2e, "unable to rename '%s' to '%s'", (string)patchedPath, targetPath);
 		}
 		return true;
 	}
 	catch (...)
 	{
-		fail();
+		warn2(__("%s: failed to proceed"), baseLongAlias);
 		return false;
 	}
 }
@@ -605,11 +585,11 @@ bool MetadataWorker::__download_index(download::Manager& downloadManager,
 	}
 
 	std::function< string () > uncompressingSub;
-	if (__is_diff_type(indexType) || indexType == IndexType::Localization)
+	if (__is_diff_type(indexType))
 	{
 		uncompressingSub = []() -> string { return ""; }; // is not a final file
 	}
-	else if (!generateUncompressingSub(uri, downloadPath, targetPath, uncompressingSub))
+	else if (!generateUncompressingSub(uri, SharedTempPath(downloadPath), targetPath, uncompressingSub))
 	{
 		return false;
 	}
@@ -661,11 +641,6 @@ bool MetadataWorker::__download_index(download::Manager& downloadManager,
 			return __download_and_apply_patches(downloadManager, downloadRecord,
 					indexEntry, baseDownloadPath, downloadPath, targetPath, _logger);
 		}
-		else if (indexType == IndexType::Localization)
-		{
-			return __download_translations(downloadManager, indexEntry,
-					uri, downloadPath, longAlias, sourceFileChanged, _logger);
-		}
 		return true;
 	}
 	else
@@ -761,9 +736,9 @@ bool MetadataWorker::__update_index(download::Manager& downloadManager, const ca
 			{
 				indexType = IndexType::PackagesDiff;
 			}
-			else if (indexType == IndexType::LocalizationFile)
+			else if (indexType == IndexType::Translation)
 			{
-				indexType = IndexType::LocalizationFileDiff;
+				indexType = IndexType::TranslationDiff;
 			}
 			else
 			{
@@ -784,148 +759,22 @@ bool MetadataWorker::__update_index(download::Manager& downloadManager, const ca
 	return false;
 }
 
-bool MetadataWorker::__download_translations(download::Manager& downloadManager,
-		const cachefiles::IndexEntry& indexEntry, const string& localizationIndexUri,
-		const string& localizationIndexPath, const string& localizationIndexLongAlias,
-		bool sourceFileChanged, Logger* logger)
-{
-	auto downloadInfo = cachefiles::getDownloadInfoOfLocalizedDescriptions2(*_config, indexEntry);
-
-	map< string, cachefiles::FileDownloadRecord > availableLocalizations;
-
-	{ // parsing localization index
-		auto cleanUp = [localizationIndexPath]()
-		{
-			unlink(localizationIndexPath.c_str());
-		};
-		auto fail = [localizationIndexLongAlias, &cleanUp]()
-		{
-			cleanUp();
-			warn2(__("failed to parse localization data from '%s'"), localizationIndexLongAlias);
-		};
-
-		try
-		{
-			string openError;
-			File localizationIndexFile(localizationIndexPath, "r", openError);
-			if (!openError.empty())
-			{
-				logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
-						piddedFormat2e, "unable to open the file '%s': %s", localizationIndexPath, openError);
-			}
-
-			TagParser localizationIndexParser(&localizationIndexFile);
-			TagParser::StringRange fieldName, fieldValue;
-			smatch m;
-			while (localizationIndexParser.parseNextLine(fieldName, fieldValue))
-			{
-				if (fieldName.equal(BUFFER_AND_SIZE("SHA1")))
-				{
-					string block;
-					localizationIndexParser.parseAdditionalLines(block);
-					auto lines = internal::split('\n', block);
-					FORIT(lineIt, lines)
-					{
-						const string& line = *lineIt;
-						if (!regex_match(line, m, checksumsLineRegex))
-						{
-							logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
-									piddedFormat2, "malformed 'hash-size-name' line '%s'", line);
-						}
-
-						cachefiles::FileDownloadRecord record;
-						record.uri = getBaseUri(localizationIndexUri) + '/' + m[3];
-						record.size = string2uint32(m[2]);
-						record.hashSums[HashSums::SHA1] = m[1];
-
-						string searchKey = m[3];
-						auto filenameExtension = getFilenameExtension(searchKey);
-						if (!filenameExtension.empty())
-						{
-							searchKey.erase(searchKey.size() - filenameExtension.size());
-						}
-
-						availableLocalizations[searchKey] = record;
-					}
-				}
-			}
-		}
-		catch (...)
-		{
-			fail();
-			return false;
-		}
-		cleanUp();
-	}
-
-	bool result = true;
-	FORIT(downloadRecordIt, downloadInfo)
-	{
-		auto it = availableLocalizations.find(downloadRecordIt->filePart);
-		if (it == availableLocalizations.end())
-		{
-			continue; // not found
-		}
-
-		const string& targetPath = downloadRecordIt->localPath;
-		if (!__download_index(downloadManager, it->second, IndexType::LocalizationFile, indexEntry,
-				getDownloadPath(targetPath), targetPath, sourceFileChanged))
-		{
-			result = false;
-		}
-	}
-
-	return result;
-}
-
 void MetadataWorker::__update_translations(download::Manager& downloadManager,
 		const cachefiles::IndexEntry& indexEntry, bool indexFileChanged)
 {
 	auto downloadInfoV3 = cachefiles::getDownloadInfoOfLocalizedDescriptions3(*_config, indexEntry);
-	if (!downloadInfoV3.empty()) // full info is available directly in Release file
+	for (const auto& record: downloadInfoV3)
 	{
-		for (const auto& record: downloadInfoV3)
-		{
-			IndexUpdateInfo info;
-			info.type = IndexType::LocalizationFile;
-			info.label = format2(__("'%s' descriptions localization"), record.language);
-			info.targetPath = record.localPath;
-			info.downloadInfo = record.fileDownloadRecords;
-			bool unused;
-			__update_index(downloadManager, indexEntry, std::move(info), indexFileChanged, unused);
-		}
-		return;
-	}
-
-	if (cachefiles::getDownloadInfoOfLocalizedDescriptions2(*_config, indexEntry).empty())
-	{
-		return;
-	}
-
-	{ // downloading translation index
-		auto localizationIndexDownloadInfo =
-				cachefiles::getDownloadInfoOfLocalizationIndex(*_config, indexEntry);
-		if (localizationIndexDownloadInfo.empty())
-		{
-			_logger->log(Logger::Subsystem::Metadata, 3,
-					__get_pidded_string("no localization file index was found in the release, skipping downloading localization files"));
-			return;
-		}
-		if (localizationIndexDownloadInfo.size() > 1)
-		{
-			_logger->log(Logger::Subsystem::Metadata, 3,
-					__get_pidded_string("more than one localization file index was found in the release, skipping downloading localization files"));
-			return;
-		}
-
-		// downloading file containing localized descriptions
-		auto baseDownloadPath = getDownloadPath(cachefiles::getPathOfIndexList(*_config, indexEntry)) + "_l10n_Index";
-		__download_index(downloadManager, localizationIndexDownloadInfo[0], IndexType::Localization,
-				indexEntry, baseDownloadPath, "" /* unused */, indexFileChanged);
+		IndexUpdateInfo info;
+		info.type = IndexType::Translation;
+		info.label = format2(__("'%s' descriptions localization"), record.language);
+		info.targetPath = record.localPath;
+		info.downloadInfo = record.fileDownloadRecords;
+		bool unused;
+		__update_index(downloadManager, indexEntry, std::move(info), indexFileChanged, unused);
 	}
 }
 
-
 bool MetadataWorker::__update_release_and_index_data(download::Manager& downloadManager,
 		const cachefiles::IndexEntry& indexEntry)
 {
